/**
\file grid.h
\date 24/04/2021
\author Merwane Bouri
\version 1
\brief Grid

Cette classe représente un réseau de cellules.

        **/

#ifndef GRID_H
#define GRID_H

#include <iostream>
#include <vector>

#include "coord.hpp"

using namespace std;

/**
\class Grid
\brief Représente une grille d'états.

Cette structure représente une grille d'états.
**/

class Structure;

class Grid{
    int nb_rows;
    int nb_col;
    std::vector<unsigned int> matrix;
public:
    //! \brief Constructeur par défaut, avec le nombre de ligne et de colonne souhaités
    Grid(size_t l,size_t c);

    //! \brief Constructeur par copie d'une grille
    Grid(const Grid& g);

    //! \brief Retourne le nombre de lignes
    //! \return Retourne le nombre de lignes de la Grille
    size_t get_rows() const {return nb_rows;}

    //! \brief Retourne le nombre de colonnes
    //! \return Retourne le nombre de colonnes de la Grille
    size_t get_col() const{return nb_col;}

    //! \brief Initialise la cellule de coordonnée pos à l'état 'state'
    void set_cell(Coord pos, unsigned int state)
    {
        int i = pos.y;
        int j = pos.x;
        matrix[((i%nb_rows + nb_rows)%nb_rows)*nb_col
               +((j%nb_col+nb_col)%nb_col)]
            =state;
    }

    //! \brief Retourne l'état d'une cellule
    //! \return Retourne l'état d'une cellule de coordonnées pos
    unsigned int get_state(Coord pos)const{
        int i = pos.y;
        int j = pos.x;
        return matrix[((i%nb_rows + nb_rows)%nb_rows)*nb_col
                      +((j%nb_col+nb_col)%nb_col)];
    }
    Grid& operator=(const Grid& g);

    //! \brief Retourne une structure représentant le contenu non-nul de la Grid (les cellules d'état != 0), pour un enregistrement dans un fichier par exemple.
    Structure to_structure() const;
};
ostream& operator<<(ostream& f, const Grid& g);

#endif // GRID_H
